@@ -17,7 +17,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: avr.c 1340 2014-11-14 10:22:52Z rliebscher $ */
+/* $Id: avr.c 1470 2021-11-12 22:02:29Z joerg_wunsch $ */
 
 #include "ac_cfg.h"
 
@@ -80,11 +80,12 @@ int avr_tpi_chip_erase(PROGRAMMER * pgm, AVRPART * p)
 			0xFF
 		};
 
-    while (avr_tpi_poll_nvmbsy(pgm));
+    while (avr_tpi_poll_nvmbsy(pgm))
+        ;
 
-		err = pgm->cmd_tpi(pgm, cmd, sizeof(cmd), NULL, 0);
-		if(err)
-			return err;
+    err = pgm->cmd_tpi(pgm, cmd, sizeof(cmd), NULL, 0);
+    if(err)
+	return err;
 
     while (avr_tpi_poll_nvmbsy(pgm));
 
@@ -329,7 +330,7 @@ int avr_read(PROGRAMMER * pgm, AVRPART * p, char * memtype,
   memset(mem->buf, 0xff, mem->size);
 
   /* supports "paged load" thru post-increment */
-  if ((p->flags & AVRPART_HAS_TPI) && mem->page_size != 0 &&
+  if ((p->flags & AVRPART_HAS_TPI) && mem->page_size > 1 &&
       pgm->cmd_tpi != NULL) {
 
     while (avr_tpi_poll_nvmbsy(pgm));
@@ -360,7 +361,7 @@ int avr_read(PROGRAMMER * pgm, AVRPART * p, char * memtype,
     return avr_mem_hiaddr(mem);
   }
 
-  if (pgm->paged_load != NULL && mem->page_size != 0) {
+  if (pgm->paged_load != NULL && mem->page_size > 1) {
     /*
      * the programmer supports a paged mode read
      */
@@ -438,11 +439,15 @@ int avr_read(PROGRAMMER * pgm, AVRPART * p, char * memtype,
     {
       rc = pgm->read_byte(pgm, p, mem, i, mem->buf + i);
       if (rc != 0) {
-	avrdude_message(MSG_INFO, "avr_read(): error reading address 0x%04lx\n", i);
-	if (rc == -1)
-	  avrdude_message(MSG_INFO, "    read operation not supported for memory \"%s\"\n",
+        avrdude_message(MSG_INFO, "avr_read(): error reading address 0x%04lx\n", i);
+        if (rc == -1) {
+          avrdude_message(MSG_INFO, "    read operation not supported for memory \"%s\"\n",
                           memtype);
-	return -2;
+          return -2;
+        }
+        avrdude_message(MSG_INFO, "    read operation failed for memory \"%s\"\n",
+                        memtype);
+        return rc;
       }
     }
     report_progress(i, mem->size, NULL);
@@ -858,7 +863,7 @@ int avr_write(PROGRAMMER * pgm, AVRPART * p, char * memtype, int size,
   }
 
 
-  if ((p->flags & AVRPART_HAS_TPI) && m->page_size != 0 &&
+  if ((p->flags & AVRPART_HAS_TPI) && m->page_size > 1 &&
       pgm->cmd_tpi != NULL) {
 
     while (avr_tpi_poll_nvmbsy(pgm));
@@ -898,7 +903,7 @@ int avr_write(PROGRAMMER * pgm, AVRPART * p, char * memtype, int size,
     return i;
   }
 
-  if (pgm->paged_write != NULL && m->page_size != 0) {
+  if (pgm->paged_write != NULL && m->page_size > 1) {
     /*
      * the programmer supports a paged mode write
      */
@@ -1051,13 +1056,37 @@ int avr_signature(PROGRAMMER * pgm, AVRPART * p)
   if (rc < 0) {
     avrdude_message(MSG_INFO, "%s: error reading signature data for part \"%s\", rc=%d\n",
                     progname, p->desc, rc);
-    return -1;
+    return rc;
   }
   report_progress (1,1,NULL);
 
   return 0;
 }
 
+static uint8_t get_fuse_bitmask(AVRMEM * m) {
+  uint8_t bitmask_r = 0;
+  uint8_t bitmask_w = 0;
+  int i;
+
+  if (!m || m->size > 1) {
+    // not a fuse, compare bytes directly
+    return 0xFF;
+  }
+
+  // For fuses, only compare bytes that are actually written *and* read.
+  for (i = 0; i < 32; i++) {
+    if (m->op[AVR_OP_WRITE]->bit[i].type == AVR_CMDBIT_INPUT)
+      bitmask_w |= (1 << m->op[AVR_OP_WRITE]->bit[i].bitno);
+    if (m->op[AVR_OP_READ]->bit[i].type == AVR_CMDBIT_OUTPUT)
+      bitmask_r |= (1 << m->op[AVR_OP_READ]->bit[i].bitno);
+  }
+  return bitmask_r & bitmask_w;
+}
+
+int compare_memory_masked(AVRMEM * m, uint8_t b1, uint8_t b2) {
+  uint8_t bitmask = get_fuse_bitmask(m);
+  return (b1 & bitmask) != (b2 & bitmask);
+}
 
 /*
  * Verify the memory buffer of p with that of v.  The byte range of v,
@@ -1104,11 +1133,30 @@ int avr_verify(AVRPART * p, AVRPART * v, char * memtype, int size)
   for (i=0; i<size; i++) {
     if ((b->tags[i] & TAG_ALLOCATED) != 0 &&
         buf1[i] != buf2[i]) {
-      avrdude_message(MSG_INFO, "%s: verification error, first mismatch at byte 0x%04x\n"
-                      "%s0x%02x != 0x%02x\n",
-                      progname, i,
-                      progbuf, buf1[i], buf2[i]);
-      return -1;
+      uint8_t bitmask = get_fuse_bitmask(a);
+      if((buf1[i] & bitmask) != (buf2[i] & bitmask)) {
+        // Mismatch is not just in unused bits
+        avrdude_message(MSG_INFO, "%s: verification error, first mismatch at byte 0x%04x\n"
+                        "%s0x%02x != 0x%02x\n",
+                        progname, i,
+                        progbuf, buf1[i], buf2[i]);
+        return -1;
+      } else {
+        // Mismatch is only in unused bits
+        if ((buf1[i] | bitmask) != 0xff) {
+          // Programmer returned unused bits as 0, must be the part/programmer
+          avrdude_message(MSG_INFO, "%s: WARNING: ignoring mismatch in unused bits of \"%s\"\n"
+                          "%s(0x%02x != 0x%02x). To prevent this warning fix the part\n"
+                          "%sor programmer definition in the config file.\n",
+                          progname, memtype, progbuf, buf1[i], buf2[i], progbuf);
+        } else {
+          // Programmer returned unused bits as 1, must be the user
+          avrdude_message(MSG_INFO, "%s: WARNING: ignoring mismatch in unused bits of \"%s\"\n"
+                          "%s(0x%02x != 0x%02x). To prevent this warning set unused bits\n"
+                          "%sto 1 when writing (double check with your datasheet first).\n",
+                          progname, memtype, progbuf, buf1[i], buf2[i], progbuf);
+        }
+      }
     }
   }
 
@@ -1191,6 +1239,16 @@ int avr_chip_erase(PROGRAMMER * pgm, AVRPART * p)
   return rc;
 }
 
+int avr_unlock(PROGRAMMER * pgm, AVRPART * p)
+{
+  int rc = -1;
+
+  if (pgm->unlock)
+    rc = pgm->unlock(pgm, p);
+
+  return rc;
+}
+
 /*
  * Report the progress of a read or write operation from/to the
  * device.
