@@ -1,7 +1,7 @@
 /*
  * avrdude - A Downloader/Uploader for AVR device programmers
  * Copyright (C) 2000, 2001, 2002, 2003  Brian S. Dean <bsd@bsdhome.com>
- * Copyright (C) 2005 Michael Holzt <kju-avr@fqdn.org>
+ * Copyright (C) 2005 Juliane Holzt <avrdude@juliane.holzt.de>
  * Copyright (C) 2011 Darell Tan <darell.tan@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-/* $Id: bitbang.c 1321 2014-06-13 20:07:40Z awachtler $ */
+/* $Id: bitbang.c 1472 2021-11-14 13:17:44Z joerg_wunsch $ */
 
 #include "ac_cfg.h"
 
@@ -542,7 +542,10 @@ int bitbang_initialize(PROGRAMMER * pgm, AVRPART * p)
 
 	/* bring RESET high first */
     pgm->setpin(pgm, PIN_AVR_RESET, 1);
-	usleep(1000);
+    usleep(128000);	/* wait t_TOUT (32-128ms) */
+
+    /* RESET must be LOW in case the existing code is driving the TPI pins: */
+    pgm->setpin(pgm, PIN_AVR_RESET, 0);
 
     avrdude_message(MSG_NOTICE2, "doing MOSI-MISO link check\n");
 
