@@ -59,17 +59,20 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <stdint.h>
+#include <math.h>
 
 #include "avrdude.h"
-#include "avr.h"
-#include "avrpart.h"
-#include "pindefs.h"
-#include "pgm.h"
-#include "config.h"
+#include "libavrdude.h"
+
 #include "bitbang.h"
 #include "ft245r.h"
 #include "usbdevs.h"
 
+#include "tpi.h"
+
+#define TPIPCR_GT_0b	0x07
+#define TPI_STOP_BITS	0x03
+
 #if defined(_WIN32)
 #include <windows.h>
 #endif
@@ -89,31 +92,11 @@
 #define DO_NOT_BUILD_FT245R
 #endif
 
-#ifndef HAVE_PTHREAD_H
-
-static int ft245r_nopthread_open (struct programmer_t *pgm, char * name) {
-    fprintf(stderr,
-            "%s: error: no pthread support. Please compile again with pthread installed."
-#if defined(_WIN32)
-            " See http://sourceware.org/pthreads-win32/."
-#endif
-            "\n",
-            progname);
-
-    return -1;
-}
-
-void ft245r_initpgm(PROGRAMMER * pgm) {
-    strcpy(pgm->type, "ftdi_syncbb");
-    pgm->open = ft245r_nopthread_open;
-}
-
-#elif defined(DO_NOT_BUILD_FT245R)
+#if defined(DO_NOT_BUILD_FT245R)
 
 static int ft245r_noftdi_open (struct programmer_t *pgm, char * name) {
-    fprintf(stderr,
-            "%s: error: no libftdi or libusb support. Install libftdi1/libusb-1.0 or libftdi/libusb and run configure/make again.\n",
-            progname);
+    avrdude_message(MSG_INFO, "%s: error: no libftdi or libusb support. Install libftdi1/libusb-1.0 or libftdi/libusb and run configure/make again.\n",
+                    progname);
 
     return -1;
 }
@@ -125,102 +108,183 @@ void ft245r_initpgm(PROGRAMMER * pgm) {
 
 #else
 
-#include <pthread.h>
-
-#ifdef __APPLE__
-/* Mac OS X defines sem_init but actually does not implement them */
-#include <dispatch/dispatch.h>
-
-typedef dispatch_semaphore_t	sem_t;
-
-#define sem_init(psem,x,val)	*psem = dispatch_semaphore_create(val)
-#define sem_post(psem)		dispatch_semaphore_signal(*psem)
-#define sem_wait(psem)		dispatch_semaphore_wait(*psem, DISPATCH_TIME_FOREVER)
-#else
-#include <semaphore.h>
-#endif
-
 #define FT245R_CYCLES	2
 #define FT245R_FRAGMENT_SIZE  512
 #define REQ_OUTSTANDINGS	10
 //#define USE_INLINE_WRITE_PAGE
 
 #define FT245R_DEBUG	0
+/*
+  Some revisions of the FTDI chips mess up the timing in bitbang mode
+  unless the bitclock is set to the max (3MHz).  For example, see:
+
+  http://www.ftdichip.com/Support/Documents/TechnicalNotes/TN_120_FT232R%20Errata%20Technical%20Note.pdf
+
+  To work around this problem, set the macro below to 1 to always set
+  the bitclock to 3MHz and then issue the same byte repeatedly to get
+  the desired timing.
+
+*/
+#define FT245R_BITBANG_VARIABLE_PULSE_WIDTH_WORKAROUND 0
 
 static struct ftdi_context *handle;
 
+#if FT245R_BITBANG_VARIABLE_PULSE_WIDTH_WORKAROUND
+static unsigned int baud_multiplier;
+#else
+# define baud_multiplier 1		// this let's C compiler optimize
+#endif
 static unsigned char ft245r_ddr;
 static unsigned char ft245r_out;
-static unsigned char ft245r_in;
 
-#define BUFSIZE 0x2000
+#define FT245R_BUFSIZE		0x2000	// receive buffer size
+#define FT245R_MIN_FIFO_SIZE	128	// min of FTDI RX/TX FIFO size
 
-// libftdi / libftd2xx compatibility functions.
+static struct {
+    int len;				// # of bytes in transmit buffer
+    uint8_t buf[FT245R_MIN_FIFO_SIZE];	// transmit buffer
+} tx;
 
-static pthread_t readerthread;
-static sem_t buf_data, buf_space;
-static unsigned char buffer[BUFSIZE];
-static int head, tail;
+static struct {
+    int discard;	// # of bytes to discard during read
+    int pending;	// # of bytes that have been written since last read
+    int wr;		// write pointer
+    int rd;		// read pointer
+    uint8_t buf[FT245R_BUFSIZE];	// receive ring buffer
+} rx;
 
-static void add_to_buf (unsigned char c) {
-    int nh;
+static int ft245r_cmd(PROGRAMMER * pgm, const unsigned char *cmd,
+                      unsigned char *res);
+static int ft245r_tpi_tx(PROGRAMMER * pgm, uint8_t byte);
+static int ft245r_tpi_rx(PROGRAMMER * pgm, uint8_t *bytep);
 
-    sem_wait (&buf_space);
-    if (head == (BUFSIZE -1)) nh = 0;
-    else                      nh = head + 1;
-
-    if (nh == tail) {
-        fprintf (stderr, "buffer overflow. Cannot happen!\n");
-        //exit (1);
-    }
-    buffer[head] = c;
-    head = nh;
-    sem_post (&buf_data);
+// Discard all data from the receive buffer.
+static void ft245r_rx_buf_purge(PROGRAMMER * pgm) {
+    rx.rd = rx.wr = 0;
 }
 
-static void *reader (void *arg) {
-    struct ftdi_context *handle = (struct ftdi_context *)(arg);
-    unsigned char buf[0x1000];
-    int br, i;
-
-    while (1) {
-        pthread_testcancel();
-        br = ftdi_read_data (handle, buf, sizeof(buf));
-        for (i=0; i<br; i++)
-            add_to_buf (buf[i]);
-    }
-    return NULL;
+static void ft245r_rx_buf_put(PROGRAMMER * pgm, uint8_t byte) {
+    rx.buf[rx.wr++] = byte;
+    if (rx.wr >= sizeof(rx.buf))
+	rx.wr = 0;
 }
 
-static int ft245r_send(PROGRAMMER * pgm, unsigned char * buf, size_t len) {
-    int rv;
+static uint8_t ft245r_rx_buf_get(PROGRAMMER * pgm) {
+    uint8_t byte = rx.buf[rx.rd++];
+    if (rx.rd >= sizeof(rx.buf))
+	rx.rd = 0;
+    return byte;
+}
 
-    rv = ftdi_write_data(handle, buf, len);
-    if (len != rv) return -1;
+/* Fill receive buffer with data from the FTDI receive FIFO.  */
+static int ft245r_fill(PROGRAMMER * pgm) {
+    uint8_t raw[FT245R_MIN_FIFO_SIZE];
+    int i, nread;
+
+    nread = ftdi_read_data(handle, raw, rx.pending);
+    if (nread < 0)
+	return -1;
+    rx.pending -= nread;
+#if FT245R_DEBUG
+    avrdude_message(MSG_INFO, "%s: read %d bytes (pending=%d)\n",
+		    __func__, nread, rx.pending);
+#endif
+    for (i = 0; i < nread; ++i)
+	ft245r_rx_buf_put(pgm, raw[i]);
+    return nread;
+}
+
+/* Flush pending TX data to the FTDI send FIFO.  */
+static int ft245r_flush(PROGRAMMER * pgm) {
+    int rv, len = tx.len, avail;
+    uint8_t *src = tx.buf;
+
+    if (!len)
+	return 0;
+
+    while (len > 0) {
+	avail = FT245R_MIN_FIFO_SIZE - rx.pending;
+	if (avail <= 0) {
+	    avail = ft245r_fill(pgm);
+	    if (avail < 0) {
+		avrdude_message(MSG_INFO,
+				"%s: fill returned %d: %s\n",
+				__func__, avail, ftdi_get_error_string(handle));
+		return -1;
+	    }
+	}
+	if (avail > len)
+	    avail = len;
+
+#if FT245R_DEBUG
+	avrdude_message(MSG_INFO, "%s: writing %d bytes\n", __func__, avail);
+#endif
+	rv = ftdi_write_data(handle, src, avail);
+	if (rv != avail) {
+	    avrdude_message(MSG_INFO,
+			    "%s: write returned %d (expected %d): %s\n",
+			    __func__, rv, avail, ftdi_get_error_string(handle));
+	    return -1;
+	}
+	src += avail;
+	len -= avail;
+	rx.pending += avail;
+    }
+    tx.len = 0;
     return 0;
 }
 
-static int ft245r_recv(PROGRAMMER * pgm, unsigned char * buf, size_t len) {
-    int i;
+static int ft245r_send2(PROGRAMMER * pgm, unsigned char * buf, size_t len,
+			bool discard_rx_data) {
+    int i, j;
 
-    // Copy over data from the circular buffer..
-    // XXX This should timeout, and return error if there isn't enough
-    // data.
-    for (i=0; i<len; i++) {
-        sem_wait (&buf_data);
-        buf[i] = buffer[tail];
-        if (tail == (BUFSIZE -1)) tail = 0;
-        else                      tail++;
-        sem_post (&buf_space);
+    for (i = 0; i < len; ++i) {
+	for (j = 0; j < baud_multiplier; ++j) {
+	    if (discard_rx_data)
+		++rx.discard;
+	    tx.buf[tx.len++] = buf[i];
+	    if (tx.len >= FT245R_MIN_FIFO_SIZE)
+		ft245r_flush(pgm);
+	}
+    }
+    return 0;
+}
+
+static int ft245r_send(PROGRAMMER * pgm, unsigned char * buf, size_t len) {
+    return ft245r_send2(pgm, buf, len, false);
+}
+
+static int ft245r_send_and_discard(PROGRAMMER * pgm, unsigned char * buf,
+				   size_t len) {
+    return ft245r_send2(pgm, buf, len, true);
+}
+
+static int ft245r_recv(PROGRAMMER * pgm, unsigned char * buf, size_t len) {
+    int i, j;
+
+    ft245r_flush(pgm);
+    ft245r_fill(pgm);
+
+#if FT245R_DEBUG
+    avrdude_message(MSG_INFO, "%s: discarding %d, consuming %zu bytes\n",
+		    __func__, rx.discard, len);
+#endif
+    while (rx.discard > 0) {
+	ft245r_rx_buf_get(pgm);
+	--rx.discard;
     }
 
+    for (i = 0; i < len; ++i) {
+	buf[i] = ft245r_rx_buf_get(pgm);
+	for (j = 1; j < baud_multiplier; ++j)
+	    ft245r_rx_buf_get(pgm);
+    }
     return 0;
 }
 
 
 static int ft245r_drain(PROGRAMMER * pgm, int display) {
     int r;
-    unsigned char t;
 
     // flush the buffer in the chip by changing the mode.....
     r = ftdi_set_bitmode(handle, 0, BITMODE_RESET); 	// reset
@@ -229,56 +293,85 @@ static int ft245r_drain(PROGRAMMER * pgm, int display) {
     if (r) return -1;
 
     // drain our buffer.
-    while (head != tail) {
-        ft245r_recv (pgm, &t, 1);
-    }
+    ft245r_rx_buf_purge(pgm);
     return 0;
 }
 
 
+/* Ensure any pending writes are sent to the FTDI chip before sleeping.  */
+static void ft245r_usleep(PROGRAMMER * pgm, useconds_t usec) {
+    ft245r_flush(pgm);
+    usleep(usec);
+}
+
+
 static int ft245r_chip_erase(PROGRAMMER * pgm, AVRPART * p) {
     unsigned char cmd[4] = {0,0,0,0};
     unsigned char res[4];
 
+    if (p->flags & AVRPART_HAS_TPI)
+      return avr_tpi_chip_erase(pgm, p);
+
     if (p->op[AVR_OP_CHIP_ERASE] == NULL) {
-        fprintf(stderr, "chip erase instruction not defined for part \"%s\"\n",
+        avrdude_message(MSG_INFO, "chip erase instruction not defined for part \"%s\"\n",
                 p->desc);
         return -1;
     }
 
     avr_set_bits(p->op[AVR_OP_CHIP_ERASE], cmd);
     pgm->cmd(pgm, cmd, res);
-    usleep(p->chip_erase_delay);
+    ft245r_usleep(pgm, p->chip_erase_delay);
     return pgm->initialize(pgm, p);
 }
 
 
 static int ft245r_set_bitclock(PROGRAMMER * pgm) {
-    int r;
-    int rate = 0;
+    // libftdi1 multiplies bitbang baudrate by 4:
+    int r, rate = 0, ftdi_rate = 3000000 / 4;
 
     /* bitclock is second. 1us = 0.000001. Max rate for ft232r 750000 */
     if(pgm->bitclock) {
-        rate = (uint32_t)(1.0/pgm->bitclock) * 2;
+        rate = (uint32_t)(1.0/pgm->bitclock);
     } else if (pgm->baudrate) {
-        rate = pgm->baudrate * 2;
+        rate = pgm->baudrate;
     } else {
         rate = 150000; /* should work for all ftdi chips and the avr default internal clock of 1MHz */
     }
 
-    if ((verbose>1) || FT245R_DEBUG) {
-        fprintf(stderr," ft245r:  spi bitclk %d -> ft baudrate %d\n",
-                rate / 2, rate);
-    }
-    r = ftdi_set_baudrate(handle, rate);
+#if FT245R_BITBANG_VARIABLE_PULSE_WIDTH_WORKAROUND
+    if (rate > 0 && rate < ftdi_rate)
+	baud_multiplier = round((ftdi_rate + rate - 1) / rate);
+    else
+	baud_multiplier = 1;
+#else
+    ftdi_rate = rate;
+#endif
+
+    avrdude_message(MSG_NOTICE2,
+		    "%s: bitclk %d -> FTDI rate %d, baud multiplier %d\n",
+		    __func__, rate, ftdi_rate, baud_multiplier);
+
+    r = ftdi_set_baudrate(handle, ftdi_rate);
     if (r) {
-        fprintf(stderr, "Set baudrate (%d) failed with error '%s'.\n",
+        avrdude_message(MSG_INFO, "Set baudrate (%d) failed with error '%s'.\n",
                 rate, ftdi_get_error_string (handle));
         return -1;
     }
     return 0;
 }
 
+static int get_pin(PROGRAMMER *pgm, int pinname) {
+  uint8_t byte;
+
+  ft245r_flush(pgm);
+
+  if (ftdi_read_pins(handle, &byte) != 0)
+    return -1;
+  if (FT245R_DEBUG)
+    avrdude_message(MSG_INFO, "%s: in 0x%02x\n", __func__, byte);
+  return GET_BITS_0(byte, pgm, pinname) != 0;
+}
+
 static int set_pin(PROGRAMMER * pgm, int pinname, int val) {
     unsigned char buf[1];
 
@@ -290,13 +383,14 @@ static int set_pin(PROGRAMMER * pgm, int pinname, int val) {
     ft245r_out = SET_BITS_0(ft245r_out,pgm,pinname,val);
     buf[0] = ft245r_out;
 
-    ft245r_send (pgm, buf, 1);
-    ft245r_recv (pgm, buf, 1);
-
-    ft245r_in = buf[0];
+    ft245r_send_and_discard(pgm, buf, 1);
     return 0;
 }
 
+static int set_sck(PROGRAMMER * pgm, int value) {
+    return set_pin(pgm, PIN_AVR_SCK, value);
+}
+
 static int set_reset(PROGRAMMER * pgm, int value) {
     return set_pin(pgm, PIN_AVR_RESET, value);
 }
@@ -334,7 +428,7 @@ static int set_led_vfy(struct programmer_t * pgm, int value) {
 static void ft245r_powerup(PROGRAMMER * pgm)
 {
     set_vcc(pgm, ON); /* power up */
-    usleep(100);
+    ft245r_usleep(pgm, 100);
 }
 
 
@@ -364,12 +458,10 @@ static void ft245r_enable(PROGRAMMER * pgm) {
    * and not via the buffer chip.
    */
     set_reset(pgm, OFF);
-    usleep(1);
+    ft245r_usleep(pgm, 1);
     set_buff(pgm, ON);
 }
 
-static int ft245r_cmd(PROGRAMMER * pgm, const unsigned char *cmd,
-                      unsigned char *res);
 /*
  * issue the 'program enable' command to the AVR device
  */
@@ -378,9 +470,12 @@ static int ft245r_program_enable(PROGRAMMER * pgm, AVRPART * p) {
     unsigned char res[4];
     int i;
 
+    if (p->flags & AVRPART_HAS_TPI)
+      return avr_tpi_program_enable(pgm, p, TPIPCR_GT_0b);
+
     if (p->op[AVR_OP_PGM_ENABLE] == NULL) {
-        fprintf(stderr,
-                "%s: AVR_OP_PGM_ENABLE command not defined for %s\n", progname, p->desc);
+        avrdude_message(MSG_INFO, "%s: AVR_OP_PGM_ENABLE command not defined for %s\n",
+                        progname, p->desc);
         fflush(stderr);
         return -1;
     }
@@ -392,23 +487,23 @@ static int ft245r_program_enable(PROGRAMMER * pgm, AVRPART * p) {
 
         if (res[p->pollindex-1] == p->pollvalue) return 0;
 
-        if ((verbose>=1) || FT245R_DEBUG) {
-            fprintf(stderr,
-                    "%s: Program enable command not successful. Retrying.\n", progname);
+        if (FT245R_DEBUG) {
+            avrdude_message(MSG_NOTICE, "%s: Program enable command not successful. Retrying.\n",
+                            progname);
             fflush(stderr);
         }
         set_pin(pgm, PIN_AVR_RESET, ON);
-        usleep(20);
+        ft245r_usleep(pgm, 20);
         set_pin(pgm, PIN_AVR_RESET, OFF);
 
         if (i == 3) {
             ft245r_drain(pgm, 0);
-            tail = head;
+	    ft245r_rx_buf_purge(pgm);
         }
     }
 
-    fprintf(stderr,
-            "%s: Device is not responding to program enable. Check connection.\n", progname);
+    avrdude_message(MSG_INFO, "%s: Device is not responding to program enable. Check connection.\n",
+                    progname);
     fflush(stderr);
 
     return -1;
@@ -419,34 +514,92 @@ static int ft245r_program_enable(PROGRAMMER * pgm, AVRPART * p) {
  */
 static int ft245r_initialize(PROGRAMMER * pgm, AVRPART * p) {
 
+    /* Apply power between VCC and GND while RESET and SCK are set to “0”. In some systems,
+     * the programmer can not guarantee that SCK is held low during power-up. In this
+     * case, RESET must be given a positive pulse of at least two CPU clock cycles duration
+     * after SCK has been set to “0”.
+     */
+    set_sck(pgm, OFF);
     ft245r_powerup(pgm);
 
     set_reset(pgm, OFF);
-    usleep(5000); // 5ms
+    ft245r_usleep(pgm, 5000); // 5ms
     set_reset(pgm, ON);
-    usleep(5000); // 5ms
+    ft245r_usleep(pgm, 5000); // 5ms
     set_reset(pgm, OFF);
-    usleep(5000); // 5ms
 
+    /* Wait for at least 20 ms and enable serial programming by sending the Programming
+     * Enable serial instruction to pin MOSI.
+     */
+    ft245r_usleep(pgm, 20000); // 20ms
+
+    if (p->flags & AVRPART_HAS_TPI) {
+	bool io_link_ok = true;
+	uint8_t byte;
+	int i;
+
+	/* Since there is a single TPIDATA line, MOSI and MISO must be
+	   linked together through a 1kOhm resistor.  Verify that
+	   everything we send on MOSI gets mirrored back on MISO.  */
+	set_pin(pgm, PIN_AVR_MOSI, 0);
+	if (get_pin(pgm, PIN_AVR_MISO) != 0) {
+	    io_link_ok = false;
+	    avrdude_message(MSG_INFO, "MOSI->MISO 0 failed\n");
+	    if (!ovsigck)
+		return -1;
+	}
+	set_pin(pgm, PIN_AVR_MOSI, 1);
+	if (get_pin(pgm, PIN_AVR_MISO) != 1) {
+	    io_link_ok = false;
+	    avrdude_message(MSG_INFO, "MOSI->MISO 1 failed\n");
+	    if (!ovsigck)
+		return -1;
+	}
+
+	if (io_link_ok)
+	    avrdude_message(MSG_NOTICE2, "MOSI-MISO link present\n");
+
+	/* keep TPIDATA high for 16 clock cycles */
+	set_pin(pgm, PIN_AVR_MOSI, 1);
+	for (i = 0; i < 16; i++) {
+	    set_sck(pgm, 1);
+	    set_sck(pgm, 0);
+	}
+
+	/* remove extra guard timing bits */
+	ft245r_tpi_tx(pgm, TPI_CMD_SSTCS | TPI_REG_TPIPCR);
+	ft245r_tpi_tx(pgm, 0x7);
+
+	/* read TPI ident reg */
+	ft245r_tpi_tx(pgm, TPI_CMD_SLDCS | TPI_REG_TPIIR);
+	ft245r_tpi_rx(pgm, &byte);
+	if (byte != 0x80) {
+	    avrdude_message(MSG_INFO, "TPIIR 0x%02x not correct\n", byte);
+	    return -1;
+	}
+    }
     return ft245r_program_enable(pgm, p);
 }
 
+static inline void add_bit(PROGRAMMER * pgm, unsigned char *buf, int *buf_pos,
+			   uint8_t bit) {
+    ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_AVR_MOSI, bit);
+    ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_AVR_SCK,0);
+    buf[*buf_pos] = ft245r_out;
+    (*buf_pos)++;
+
+    ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_AVR_SCK,1);
+    buf[*buf_pos] = ft245r_out;
+    (*buf_pos)++;
+}
+
 static inline int set_data(PROGRAMMER * pgm, unsigned char *buf, unsigned char data) {
     int j;
     int buf_pos = 0;
     unsigned char bit = 0x80;
 
     for (j=0; j<8; j++) {
-        ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_AVR_MOSI,data & bit);
-
-        ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_AVR_SCK,0);
-        buf[buf_pos] = ft245r_out;
-        buf_pos++;
-
-        ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_AVR_SCK,1);
-        buf[buf_pos] = ft245r_out;
-        buf_pos++;
-
+	add_bit(pgm, buf, &buf_pos, (data & bit) != 0);
         bit >>= 1;
     }
     return buf_pos;
@@ -454,7 +607,8 @@ static inline int set_data(PROGRAMMER * pgm, unsigned char *buf, unsigned char d
 
 static inline unsigned char extract_data(PROGRAMMER * pgm, unsigned char *buf, int offset) {
     int j;
-    int buf_pos = 1;
+    int buf_pos = FT245R_CYCLES; /* MISO data is valid AFTER rising SCK edge,
+                                            i.e. in next clock cycle */
     unsigned char bit = 0x80;
     unsigned char r = 0;
 
@@ -470,6 +624,7 @@ static inline unsigned char extract_data(PROGRAMMER * pgm, unsigned char *buf, i
 }
 
 /* to check data */
+#if 0
 static inline unsigned char extract_data_out(PROGRAMMER * pgm, unsigned char *buf, int offset) {
     int j;
     int buf_pos = 1;
@@ -486,6 +641,7 @@ static inline unsigned char extract_data_out(PROGRAMMER * pgm, unsigned char *bu
     }
     return r;
 }
+#endif
 
 
 /*
@@ -514,6 +670,122 @@ static int ft245r_cmd(PROGRAMMER * pgm, const unsigned char *cmd,
     return 0;
 }
 
+static inline uint8_t extract_tpi_data(PROGRAMMER * pgm, unsigned char *buf,
+				       int *buf_pos) {
+    uint8_t bit = 0x1, byte = 0;
+    int j;
+
+    for (j = 0; j < 8; j++) {
+	(*buf_pos)++;	// skip over falling clock edge
+        if (GET_BITS_0(buf[(*buf_pos)++], pgm, PIN_AVR_MISO))
+            byte |= bit;
+        bit <<= 1;
+    }
+    return byte;
+}
+
+static inline int set_tpi_data(PROGRAMMER * pgm, unsigned char *buf,
+			       uint8_t byte) {
+    uint8_t bit = 0x1, parity = 0;
+    int j, buf_pos = 0;
+
+    // start bit:
+    add_bit(pgm, buf, &buf_pos, 0);
+
+    // 8 data bits:
+    for (j = 0; j < 8; j++) {
+	add_bit(pgm, buf, &buf_pos, (byte & bit) != 0);
+	parity ^= (byte & bit) != 0;
+        bit <<= 1;
+    }
+
+    // parity bit:
+    add_bit(pgm, buf, &buf_pos, parity);
+    // stop bits:
+    add_bit(pgm, buf, &buf_pos, 1);
+    add_bit(pgm, buf, &buf_pos, 1);
+    return buf_pos;
+}
+
+static int ft245r_tpi_tx(PROGRAMMER * pgm, uint8_t byte) {
+    uint8_t buf[128];
+    int len;
+
+    len = set_tpi_data(pgm, buf, byte);
+    ft245r_send_and_discard(pgm, buf, len);
+    return 0;
+}
+
+static int ft245r_tpi_rx(PROGRAMMER * pgm, uint8_t *bytep) {
+    uint8_t buf[128], bit, parity;
+    int i, buf_pos = 0, len = 0;
+    uint32_t res, m, byte;
+
+    /* Allow for up to 4 bits before we must see start bit; during
+       that time, we must keep the MOSI line high. */
+    for (i = 0; i < 2; ++i)
+	len += set_data(pgm, &buf[len], 0xff);
+
+    ft245r_send(pgm, buf, len);
+    ft245r_recv(pgm, buf, len);
+
+    res = (extract_tpi_data(pgm, buf, &buf_pos)
+	   | ((uint32_t) extract_tpi_data(pgm, buf, &buf_pos) << 8));
+
+    /* Look for start bit: */
+    m = 0x1;
+    while (m & res)
+	m <<= 1;
+    if (m >= 0x10) {
+	avrdude_message(MSG_INFO, "%s: start bit missing (res=0x%04x)\n",
+			__func__, res);
+	return -1;
+    }
+    byte = parity = 0;
+    for (i = 0; i < 8; ++i) {
+	m <<= 1;
+	bit = (res & m) != 0;
+	parity ^= bit;
+	byte |= bit << i;
+    }
+    m <<= 1;
+    if (((res & m) != 0) != parity) {
+	avrdude_message(MSG_INFO, "%s: parity bit wrong\n", __func__);
+	return -1;
+    }
+    if (((res & (m << 1)) == 0) || ((res & (m << 2))) == 0) {
+	avrdude_message(MSG_INFO, "%s: stop bits wrong\n", __func__);
+	return -1;
+    }
+    *bytep = (uint8_t) byte;
+    return 0;
+}
+
+static int ft245r_cmd_tpi(PROGRAMMER * pgm, const unsigned char *cmd,
+			  int cmd_len, unsigned char *res, int res_len) {
+    int i, ret = 0;
+
+    pgm->pgm_led(pgm, ON);
+
+    for (i = 0; i < cmd_len; ++i)
+	ft245r_tpi_tx(pgm, cmd[i]);
+    for (i = 0; i < res_len; ++i)
+	if ((ret = ft245r_tpi_rx(pgm, &res[i])) < 0)
+	    break;
+    if (verbose >= 2) {
+	avrdude_message(MSG_NOTICE2, "%s: [ ", __func__);
+	for (i = 0; i < cmd_len; i++)
+	    avrdude_message(MSG_NOTICE2, "%02X ", cmd[i]);
+	avrdude_message(MSG_NOTICE2, "] [ ");
+	for(i = 0; i < res_len; i++)
+	    avrdude_message(MSG_NOTICE2, "%02X ", res[i]);
+	avrdude_message(MSG_NOTICE2, "]\n");
+    }
+
+    pgm->pgm_led(pgm, OFF);
+    return ret;
+}
+
 /* lower 8 pins are accepted, they might be also inverted */
 static const struct pindef_t valid_pins = {{0xff},{0xff}} ;
 
@@ -528,8 +800,10 @@ static const struct pin_checklist_t pin_checklist[] = {
 static int ft245r_open(PROGRAMMER * pgm, char * port) {
     int rv;
     int devnum = -1;
+    char device[9] = "";
 
     rv = pins_check(pgm,pin_checklist,sizeof(pin_checklist)/sizeof(pin_checklist[0]), true);
+
     if(rv) {
         pgm->display(pgm, progbuf);
         return rv;
@@ -537,23 +811,47 @@ static int ft245r_open(PROGRAMMER * pgm, char * port) {
 
     strcpy(pgm->port, port);
 
-    if (strcmp(port,DEFAULT_USB) != 0) {
-        if (strncasecmp("ft", port, 2) == 0) {
-            char *startptr = port + 2;
-            char *endptr = NULL;
-            devnum = strtol(startptr,&endptr,10);
-            if ((startptr==endptr) || (*endptr != '\0')) {
-                devnum = -1;
-            }
-        }
-        if (devnum < 0) {
-            fprintf(stderr,
-                    "%s: invalid portname '%s': use 'ft[0-9]+'\n",
-                    progname,port);
-            return -1;
-        }
+    // read device string cut after 8 chars (max. length of serial number)
+    if ((sscanf(port, "usb:%8s", device) != 1)) {
+      avrdude_message(MSG_NOTICE,
+          "%s: ft245r_open(): no device identifier in portname, using default\n",
+          progname);
+      pgm->usbsn[0] = 0;
+      devnum = 0;
     } else {
+      if (strlen(device) == 8 ){ // serial number
+        if (verbose >= 2) {
+          avrdude_message(MSG_INFO,
+              "%s: ft245r_open(): serial number parsed as: "
+              "%s\n",
+              progname,
+              device);
+        }
+        // copy serial number to pgm struct
+        strcpy(pgm->usbsn, device);
+        // and use first device with matching serial (should be unique)
         devnum = 0;
+      }
+      else if (strncmp("ft", device, 2) || strlen(device) <= 8)  { // classic device number
+        char *startptr = device + 2;
+        char *endptr = NULL;
+        devnum = strtol(startptr,&endptr,10);
+        if ((startptr==endptr) || (*endptr != '\0')) {
+          devnum = -1;
+        }
+        avrdude_message(MSG_INFO,
+            "%s: ft245r_open(): device number parsed as: "
+            "%d\n",
+            progname,
+            devnum);
+      }
+    }
+
+    // if something went wrong before abort with helpful message
+    if (devnum < 0) {
+      avrdude_message(MSG_INFO, "%s: ft245r_open(): invalid portname '%s': use^ 'ft[0-9]+' or serial number\n",
+          progname,port);
+      return -1;
     }
 
     handle = malloc (sizeof (struct ftdi_context));
@@ -563,8 +861,7 @@ static int ft245r_open(PROGRAMMER * pgm, char * port) {
     if (usbpid) {
       pid = *(int *)(ldata(usbpid));
       if (lnext(usbpid))
-	fprintf(stderr,
-		"%s: Warning: using PID 0x%04x, ignoring remaining PIDs in list\n",
+	avrdude_message(MSG_INFO, "%s: Warning: using PID 0x%04x, ignoring remaining PIDs in list\n",
 		progname, pid);
     } else {
       pid = USB_DEVICE_FT245;
@@ -576,7 +873,8 @@ static int ft245r_open(PROGRAMMER * pgm, char * port) {
                                   pgm->usbsn[0]?pgm->usbsn:NULL,
                                   devnum);
     if (rv) {
-        fprintf (stderr, "can't open ftdi device %d. (%s)\n", devnum, ftdi_get_error_string(handle));
+        avrdude_message(MSG_INFO, "%s: can't open ftdi device: %s\n",
+                        progname, ftdi_get_error_string(handle));
         goto cleanup_no_usb;
     }
 
@@ -604,11 +902,17 @@ static int ft245r_open(PROGRAMMER * pgm, char * port) {
     ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_LED_VFY,0);
 
 
+    rv = ftdi_set_latency_timer(handle, 1);
+    if (rv) {
+        avrdude_message(MSG_INFO, "%s: unable to set latency timer to 1 (%s)\n",
+                        progname, ftdi_get_error_string(handle));
+        goto cleanup;
+    }
+
     rv = ftdi_set_bitmode(handle, ft245r_ddr, BITMODE_SYNCBB); // set Synchronous BitBang
     if (rv) {
-        fprintf(stderr,
-                "%s: Synchronous BitBangMode is not supported (%s)\n",
-                progname, ftdi_get_error_string(handle));
+        avrdude_message(MSG_INFO, "%s: Synchronous BitBangMode is not supported (%s)\n",
+                        progname, ftdi_get_error_string(handle));
         goto cleanup;
     }
 
@@ -617,22 +921,12 @@ static int ft245r_open(PROGRAMMER * pgm, char * port) {
         goto cleanup;
     }
 
-    /* We start a new thread to read the output from the FTDI. This is
-     * necessary because otherwise we'll deadlock. We cannot finish
-     * writing because the ftdi cannot send the results because we
-     * haven't provided a read buffer yet. */
-
-    sem_init (&buf_data, 0, 0);
-    sem_init (&buf_space, 0, BUFSIZE);
-    pthread_create (&readerthread, NULL, reader, handle);
-
     /*
      * drain any extraneous input
      */
     ft245r_drain (pgm, 0);
 
-    ft245r_send (pgm, &ft245r_out, 1);
-    ft245r_recv (pgm, &ft245r_in, 1);
+    ft245r_send_and_discard(pgm, &ft245r_out, 1);
 
     return 0;
 
@@ -651,17 +945,15 @@ static void ft245r_close(PROGRAMMER * pgm) {
         // I think the switch to BB mode and back flushes the buffer.
         ftdi_set_bitmode(handle, 0, BITMODE_SYNCBB); // set Synchronous BitBang, all in puts
         ftdi_set_bitmode(handle, 0, BITMODE_RESET); // disable Synchronous BitBang
-        pthread_cancel(readerthread);
-        pthread_join(readerthread, NULL);
         ftdi_usb_close(handle);
-        ftdi_deinit (handle); // TODO this works with libftdi 0.20, but hangs with 1.0
+        ftdi_deinit (handle);
         free(handle);
         handle = NULL;
     }
 }
 
 static void ft245r_display(PROGRAMMER * pgm, const char * p) {
-    fprintf(stderr, "%sPin assignment  : 0..7 = DBUS0..7\n",p);/* , 8..11 = GPIO0..3\n",p);*/
+    avrdude_message(MSG_INFO, "%sPin assignment  : 0..7 = DBUS0..7\n",p);/* , 8..11 = GPIO0..3\n",p);*/
     pgm_display_generic_mask(pgm, p, SHOW_ALL_PINS);
 }
 
@@ -714,7 +1006,7 @@ static void put_request(int addr, int bytes, int n) {
     } else {
         p = malloc(sizeof(struct ft245r_request));
         if (!p) {
-            fprintf(stderr, "can't alloc memory\n");
+            avrdude_message(MSG_INFO, "can't alloc memory\n");
             exit(1);
         }
     }
@@ -804,11 +1096,16 @@ static int ft245r_paged_write_flash(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
             ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_AVR_SCK,0); // sck down
             buf[buf_pos++] = ft245r_out;
         }
+        else {
+            /* stretch sequence to allow correct readout, see extract_data() */
+            buf[buf_pos] = buf[buf_pos - 1];
+            buf_pos++;
+        }
         ft245r_send(pgm, buf, buf_pos);
         put_request(addr_save, buf_pos, 0);
         //ft245r_sync(pgm);
 #if 0
-        fprintf(stderr, "send addr 0x%04x bufsize %d [%02x %02x] page_write %d\n",
+        avrdude_message(MSG_INFO, "send addr 0x%04x bufsize %d [%02x %02x] page_write %d\n",
                 addr_save,buf_pos,
                 extract_data_out(pgm, buf , (0*4 + 3) ),
                 extract_data_out(pgm, buf , (1*4 + 3) ),
@@ -821,7 +1118,7 @@ static int ft245r_paged_write_flash(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
 #if defined(USE_INLINE_WRITE_PAGE)
             while (do_request(pgm, m))
                 ;
-            usleep(m->max_write_delay);
+            ft245r_usleep(pgm, m->max_write_delay);
 #else
             int addr_wk = addr_save - (addr_save % m->page_size);
             int rc;
@@ -893,6 +1190,11 @@ static int ft245r_paged_load_flash(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
             ft245r_out = SET_BITS_0(ft245r_out,pgm,PIN_AVR_SCK,0); // sck down
             buf[buf_pos++] = ft245r_out;
         }
+        else {
+            /* stretch sequence to allow correct readout, see extract_data() */
+            buf[buf_pos] = buf[buf_pos - 1];
+            buf_pos++;
+        }
         n = j;
         ft245r_send(pgm, buf, buf_pos);
         put_request(addr_save, buf_pos, n);
@@ -931,6 +1233,7 @@ void ft245r_initpgm(PROGRAMMER * pgm) {
     pgm->program_enable = ft245r_program_enable;
     pgm->chip_erase     = ft245r_chip_erase;
     pgm->cmd            = ft245r_cmd;
+    pgm->cmd_tpi        = ft245r_cmd_tpi;
     pgm->open           = ft245r_open;
     pgm->close          = ft245r_close;
     pgm->read_byte      = avr_read_byte_default;
