@@ -1,7 +1,7 @@
 /*
  * avrdude - A Downloader/Uploader for AVR device programmers
  * Copyright (C) 2000-2005  Brian S. Dean <bsd@bsdhome.com>
- * Copyright 2007-2014 Joerg Wunsch <j@uriah.heep.sax.de>
+ * Copyright Joerg Wunsch <j@uriah.heep.sax.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: main.c 1347 2014-11-23 21:49:56Z joerg_wunsch $ */
+/* $Id: main.c 1523 2021-12-16 21:02:35Z joerg_wunsch $ */
 
 /*
  * Code to program an Atmel AVR device through one of the supported
@@ -127,8 +127,6 @@ static void usage(void)
  "  -t                         Enter terminal mode.\n"
  "  -E <exitspec>[,<exitspec>] List programmer exit specifications.\n"
  "  -x <extended_param>        Pass <extended_param> to programmer.\n"
- "  -y                         Count # erase cycles in EEPROM.\n"
- "  -Y <number>                Initialize erase cycle # in EEPROM.\n"
  "  -v                         Verbose output. -v -v for more.\n"
  "  -q                         Quell progress output. -q -q for less.\n"
  "  -l logfile                 Use logfile rather than stderr for diagnostics.\n"
@@ -347,7 +345,6 @@ int main(int argc, char * argv [])
   unsigned char safemode_fuse = 0xff;
 
   char * safemode_response;
-  int fuses_specified = 0;
   int fuses_updated = 0;
 #if !defined(WIN32NATIVE)
   char  * homedir;
@@ -679,10 +676,10 @@ int main(int argc, char * argv [])
    * Print out an identifying string so folks can tell what version
    * they are running
    */
-  avrdude_message(MSG_NOTICE, "\n%s: Version %s, compiled on %s at %s\n"
-                    "%sCopyright (c) 2000-2005 Brian Dean, http://www.bdmicro.com/\n"
-                    "%sCopyright (c) 2007-2014 Joerg Wunsch\n\n",
-                    progname, version, __DATE__, __TIME__, progbuf, progbuf);
+  avrdude_message(MSG_NOTICE, "\n%s: Version %s\n"
+                    "%sCopyright (c) Brian Dean, http://www.bdmicro.com/\n"
+                    "%sCopyright (c) Joerg Wunsch\n\n",
+                    progname, version, progbuf, progbuf);
   avrdude_message(MSG_NOTICE, "%sSystem wide configuration file is \"%s\"\n",
             progbuf, sys_config);
 
@@ -921,7 +918,7 @@ int main(int argc, char * argv [])
    */
   if (port[0] == 0) {
     avrdude_message(MSG_INFO, "\n%s: no port has been specified on the command line "
-            "or the config file\n",
+            "or in the config file\n",
             progname);
     avrdude_message(MSG_INFO, "%sSpecify a port using the -P option and try again\n\n",
             progbuf);
@@ -1045,6 +1042,44 @@ int main(int argc, char * argv [])
     if (init_ok) {
       rc = avr_signature(pgm, p);
       if (rc != 0) {
+        // -68 == -(0x44) == -(RSP3_FAIL_OCD_LOCKED)
+        if ((rc == -68) && (p->flags & AVRPART_HAS_UPDI) && (attempt < 1)) {
+          attempt++;
+          if (pgm->read_sib) {
+             // Read SIB and compare FamilyID
+             char sib[AVR_SIBLEN + 1];
+             pgm->read_sib(pgm, p, sib);
+             avrdude_message(MSG_NOTICE, "%s: System Information Block: \"%s\"\n",
+                             progname, sib);
+            if (quell_progress < 2) {
+              avrdude_message(MSG_INFO, "%s: Received FamilyID: \"%.*s\"\n", progname, AVR_FAMILYIDLEN, sib);
+            }
+            if (strncmp(p->family_id, sib, AVR_FAMILYIDLEN)) {
+              avrdude_message(MSG_INFO, "%s: Expected FamilyID: \"%s\"\n", progname, p->family_id);
+              if (!ovsigck) {
+                avrdude_message(MSG_INFO, "%sDouble check chip, "
+                        "or use -F to override this check.\n",
+                        progbuf);
+                exitrc = 1;
+                goto main_exit;
+              }
+            }
+          }
+          if(erase) {
+            erase = 0;
+            if (uflags & UF_NOWRITE) {
+              avrdude_message(MSG_INFO, "%s: conflicting -e and -n options specified, NOT erasing chip\n",
+                              progname);
+            } else {
+              if (quell_progress < 2) {
+                avrdude_message(MSG_INFO, "%s: erasing chip\n", progname);
+              }
+              exitrc = avr_unlock(pgm, p);
+              if(exitrc) goto main_exit;
+              goto sig_again;
+            }
+          }
+        }
         avrdude_message(MSG_INFO, "%s: error reading signature data, rc=%d\n",
           progname, rc);
         exitrc = 1;
@@ -1138,7 +1173,7 @@ int main(int argc, char * argv [])
 	  //Check if the programmer just doesn't support reading
   	  if (rc == -5)
 			{
-				avrdude_message(MSG_NOTICE, "%s: safemode: Fuse reading not support by programmer.\n"
+				avrdude_message(MSG_NOTICE, "%s: safemode: Fuse reading not supported by programmer.\n"
 						"              Safemode disabled.\n", progname);
 			}
       else
@@ -1264,8 +1299,11 @@ int main(int argc, char * argv [])
       }
     }
     
+    AVRMEM * m;
+    
     /* Now check what fuses are against what they should be */
-    if (safemodeafter_fuse != safemode_fuse) {
+    m = avr_locate_mem(p, "fuse");
+    if (compare_memory_masked(m, safemodeafter_fuse, safemode_fuse)) {
       fuses_updated = 1;
       avrdude_message(MSG_INFO, "%s: safemode: fuse changed! Was %x, and is now %x\n",
               progname, safemode_fuse, safemodeafter_fuse);
@@ -1293,7 +1331,8 @@ int main(int argc, char * argv [])
     }
 
     /* Now check what fuses are against what they should be */
-    if (safemodeafter_lfuse != safemode_lfuse) {
+    m = avr_locate_mem(p, "lfuse");
+    if (compare_memory_masked(m, safemodeafter_lfuse, safemode_lfuse)) {
       fuses_updated = 1;
       avrdude_message(MSG_INFO, "%s: safemode: lfuse changed! Was %x, and is now %x\n",
               progname, safemode_lfuse, safemodeafter_lfuse);
@@ -1321,7 +1360,8 @@ int main(int argc, char * argv [])
     }
 
     /* Now check what fuses are against what they should be */
-    if (safemodeafter_hfuse != safemode_hfuse) {
+    m = avr_locate_mem(p, "hfuse");
+    if (compare_memory_masked(m, safemodeafter_hfuse, safemode_hfuse)) {
       fuses_updated = 1;
       avrdude_message(MSG_INFO, "%s: safemode: hfuse changed! Was %x, and is now %x\n",
               progname, safemode_hfuse, safemodeafter_hfuse);
@@ -1346,7 +1386,8 @@ int main(int argc, char * argv [])
     }
 
     /* Now check what fuses are against what they should be */
-    if (safemodeafter_efuse != safemode_efuse) {
+    m = avr_locate_mem(p, "efuse");
+    if (compare_memory_masked(m, safemodeafter_efuse, safemode_efuse)) {
       fuses_updated = 1;
       avrdude_message(MSG_INFO, "%s: safemode: efuse changed! Was %x, and is now %x\n",
               progname, safemode_efuse, safemodeafter_efuse);
@@ -1374,14 +1415,14 @@ int main(int argc, char * argv [])
       avrdude_message(MSG_INFO, "%s: safemode: ", progname);
       if (failures == 0) {
         avrdude_message(MSG_INFO, "Fuses OK (E:%02X, H:%02X, L:%02X)\n",
-                safemode_efuse, safemode_hfuse, safemode_lfuse);
+                safemodeafter_efuse, safemodeafter_hfuse, safemodeafter_lfuse);
       }
       else {
         avrdude_message(MSG_INFO, "Fuses not recovered, sorry\n");
       }
     }
 
-    if (fuses_updated && fuses_specified) {
+    if (fuses_updated) {
       exitrc = 1;
     }
 
