@@ -17,7 +17,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: usbasp.c 1343 2014-11-20 09:20:44Z joerg_wunsch $ */
+/* $Id: usbasp.c 1468 2021-11-07 20:10:24Z joerg_wunsch $ */
 
 /*
  * Interface to the USBasp programmer.
@@ -120,6 +120,8 @@ struct pdata
   int sckfreq_hz;
   unsigned int capabilities;
   int use_tpi;
+  int section_e;
+  int sck_3mhz;
 };
 
 #define PDATA(pgm) ((struct pdata *)(pgm->cookie))
@@ -131,6 +133,7 @@ struct pdata
 // interface - management
 static void usbasp_setup(PROGRAMMER * pgm);
 static void usbasp_teardown(PROGRAMMER * pgm);
+static int usbasp_parseextparms(PROGRAMMER * pgm, LISTID extparms);
 // internal functions
 static int usbasp_transmit(PROGRAMMER * pgm, unsigned char receive,
 			   unsigned char functionid, const unsigned char *send,
@@ -192,6 +195,31 @@ static void usbasp_teardown(PROGRAMMER * pgm)
   free(pgm->cookie);
 }
 
+static int usbasp_parseextparms(PROGRAMMER * pgm, LISTID extparms)
+{
+  LNODEID ln;
+  const char *extended_param;
+  int rv = 0;
+
+  for (ln = lfirst(extparms); ln; ln = lnext(ln)) {
+    extended_param = ldata(ln);
+
+    if (strncmp(extended_param, "section_config", strlen("section_config")) == 0) {
+      avrdude_message(MSG_NOTICE2, "%s: usbasp_parseextparms(): set section_e to 1 (config section)\n",
+                      progname);
+      PDATA(pgm)->section_e = 1;
+      continue;
+    }
+
+    avrdude_message(MSG_INFO, "%s: usbasp_parseextparms(): invalid extended parameter '%s'\n",
+                    progname, extended_param);
+    rv = -1;
+  }
+
+  return rv;
+}
+
+
 /* Internal functions */
 
 static const char *usbasp_get_funcname(unsigned char functionid)
@@ -570,6 +598,9 @@ static int usbasp_initialize(PROGRAMMER * pgm, AVRPART * p)
     pdata->capabilities = 0;
 
   pdata->use_tpi = ((pdata->capabilities & USBASP_CAP_TPI) != 0 && (p->flags & AVRPART_HAS_TPI) != 0) ? 1 : 0;
+  // query support for 3 MHz SCK in UsbAsp-flash firmware
+  // https://github.com/nofeletru/UsbAsp-flash
+  pdata->sck_3mhz = ((pdata->capabilities & USBASP_CAP_3MHZ) != 0) ? 1 :0;
 
   if(pdata->use_tpi)
   {
@@ -635,7 +666,7 @@ static int usbasp_spi_cmd(PROGRAMMER * pgm, const unsigned char *cmd,
     if (verbose == 3)
       putc('\n', stderr);
 
-    avrdude_message(MSG_INFO, "%s: error: wrong responds size\n",
+    avrdude_message(MSG_INFO, "%s: error: wrong response size\n",
 	    progname);
     return -1;
   }
@@ -836,6 +867,7 @@ static int usbasp_spi_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
 
 /* The list of SCK frequencies in Hz supported by USBasp */
 static struct sckoptions_t usbaspSCKoptions[] = {
+  { USBASP_ISP_SCK_3000, 3000000 },
   { USBASP_ISP_SCK_1500, 1500000 },
   { USBASP_ISP_SCK_750, 750000 },
   { USBASP_ISP_SCK_375, 375000 },
@@ -881,14 +913,23 @@ static int usbasp_spi_set_sck_period(PROGRAMMER *pgm, double sckperiod)
 
     avrdude_message(MSG_NOTICE2, "%s: try to set SCK period to %g s (= %i Hz)\n", progname, sckperiod, sckfreq);
 
-    if (sckfreq >= usbaspSCKoptions[0].frequency) {
-      clockoption = usbaspSCKoptions[0].id;
-      usefreq = usbaspSCKoptions[0].frequency;
+    /* Check if programmer is capable of 3 MHz SCK, if not then ommit 3 MHz setting */
+    int i;
+    if (PDATA(pgm)->sck_3mhz) {
+      avrdude_message(MSG_NOTICE2, "%s: connected USBasp is capable of 3 MHz SCK\n",progname);
+      i = 0;
+    } else {
+      avrdude_message(MSG_NOTICE2, "%s: connected USBasp is not cabable of 3 MHz SCK\n",progname);
+      i = 1;
+    }
+    if (sckfreq >= usbaspSCKoptions[i].frequency) {
+      clockoption = usbaspSCKoptions[i].id;
+      usefreq = usbaspSCKoptions[i].frequency;
     } else {
 
       /* find clock option next to given clock */
-      int i;
-      for (i = 0; i < sizeof(usbaspSCKoptions) / sizeof(usbaspSCKoptions[0]); i++) {
+
+      for (; i < sizeof(usbaspSCKoptions) / sizeof(usbaspSCKoptions[0]); i++) {
         if (sckfreq >= usbaspSCKoptions[i].frequency - 1) { /* subtract 1 to compensate round errors */
           clockoption = usbaspSCKoptions[i].id;
           usefreq = usbaspSCKoptions[i].frequency;
@@ -936,7 +977,7 @@ static int usbasp_tpi_recv_byte(PROGRAMMER * pgm)
 
   if(usbasp_transmit(pgm, 1, USBASP_FUNC_TPI_RAWREAD, temp, temp, sizeof(temp)) != 1)
   {
-    avrdude_message(MSG_INFO, "%s: error: wrong responds size\n", progname);
+    avrdude_message(MSG_INFO, "%s: error: wrong response size\n", progname);
     return -1;
   }
 
@@ -1015,21 +1056,40 @@ static int usbasp_tpi_program_enable(PROGRAMMER * pgm, AVRPART * p)
 
 static int usbasp_tpi_chip_erase(PROGRAMMER * pgm, AVRPART * p)
 {
-  avrdude_message(MSG_DEBUG, "%s: usbasp_tpi_chip_erase()\n", progname);
+  int pr_0;
+  int pr_1;
+  int nvm_cmd;
 
-  /* Set PR to flash */
+  switch (PDATA(pgm)->section_e) {
+    /* Config bits section erase */
+  case 1:
+    pr_0 = 0x41;
+    pr_1 = 0x3F;
+    nvm_cmd = NVMCMD_SECTION_ERASE;
+    avrdude_message(MSG_DEBUG, "%s: usbasp_tpi_chip_erase() - section erase\n", progname);
+    break;
+    /* Chip erase (flash only) */
+  default:
+    pr_0 = 0x01;
+    pr_1 = 0x40;
+    nvm_cmd = NVMCMD_CHIP_ERASE;
+    avrdude_message(MSG_DEBUG, "%s: usbasp_tpi_chip_erase() - chip erase\n", progname);
+    break;
+  }
+
+  /* Set PR */
   usbasp_tpi_send_byte(pgm, TPI_OP_SSTPR(0));
-  usbasp_tpi_send_byte(pgm, 0x01);
+  usbasp_tpi_send_byte(pgm, pr_0);
   usbasp_tpi_send_byte(pgm, TPI_OP_SSTPR(1));
-  usbasp_tpi_send_byte(pgm, 0x40);
-  /* select ERASE */
+  usbasp_tpi_send_byte(pgm, pr_1);
+  /* select what been erase  */
   usbasp_tpi_send_byte(pgm, TPI_OP_SOUT(NVMCMD));
-  usbasp_tpi_send_byte(pgm, NVMCMD_CHIP_ERASE);
+  usbasp_tpi_send_byte(pgm, nvm_cmd);
   /* dummy write */
   usbasp_tpi_send_byte(pgm, TPI_OP_SST_INC);
   usbasp_tpi_send_byte(pgm, 0x00);
   usbasp_tpi_nvm_waitbusy(pgm);
-  
+
   usleep(p->chip_erase_delay);
   pgm->initialize(pgm, p);
 
@@ -1096,6 +1156,24 @@ static int usbasp_tpi_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
   pr = addr + m->offset;
   writed = 0;
 
+  /* must erase fuse first */
+  if(strcmp(m->desc, "fuse") == 0)
+  {
+    /* Set PR */
+    usbasp_tpi_send_byte(pgm, TPI_OP_SSTPR(0));
+    usbasp_tpi_send_byte(pgm, (pr & 0xFF) | 1 );
+    usbasp_tpi_send_byte(pgm, TPI_OP_SSTPR(1));
+    usbasp_tpi_send_byte(pgm, (pr >> 8) );
+    /* select SECTION_ERASE */
+    usbasp_tpi_send_byte(pgm, TPI_OP_SOUT(NVMCMD));
+    usbasp_tpi_send_byte(pgm, NVMCMD_SECTION_ERASE);
+    /* dummy write */
+    usbasp_tpi_send_byte(pgm, TPI_OP_SST_INC);
+    usbasp_tpi_send_byte(pgm, 0x00);
+
+    usbasp_tpi_nvm_waitbusy(pgm);
+  }
+
   /* Set PR to flash */
   usbasp_tpi_send_byte(pgm, TPI_OP_SSTPR(0));
   usbasp_tpi_send_byte(pgm, (pr & 0xFF) | 1 );
@@ -1194,6 +1272,7 @@ void usbasp_initpgm(PROGRAMMER * pgm)
   pgm->setup          = usbasp_setup;
   pgm->teardown       = usbasp_teardown;
   pgm->set_sck_period = usbasp_spi_set_sck_period;
+  pgm->parseextparams = usbasp_parseextparms;
 
 }
 
