@@ -16,7 +16,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: usbdevs.h 1294 2014-03-12 23:03:18Z joerg_wunsch $ */
+/* $Id: usbdevs.h 1454 2021-11-06 22:13:51Z joerg_wunsch $ */
 
 /*
  * defines for the USB interface
@@ -69,7 +69,14 @@
 #define USBDEV_BULK_EP_WRITE_3    0x01
 #define USBDEV_BULK_EP_READ_3     0x82
 #define USBDEV_EVT_EP_READ_3      0x83
-#define USBDEV_MAX_XFER_3    512
+/* 
+ * The mk3 tools (type jtagice3) have a maxPayloadSize of 912. When
+ * accessing paged memory the access should be limited to pageSize.
+ * When accessing memory without pageSize the payload should be
+ * limited to 256.
+ */
+#define USBDEV_MAX_XFER_3         912
+#define USBDEV_MAX_XFER_3_UNPAGED 256
 
 /*
  * When operating on the JTAGICE3, usbdev_recv_frame() returns an
